package meerkat.pollingstation;

import com.google.common.util.concurrent.FutureCallback;
import meerkat.protobuf.PollingStation;
import static meerkat.rest.Constants.*;

import javax.ws.rs.*;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import static meerkat.pollingstation.PollingStationConstants.*;
import meerkat.protobuf.PollingStation.*;

/**
 * Created by Arbel on 05/05/2016.
 */
@Path(POLLING_STATION_WEB_SCANNER_PATH)
public class PollingStationWebScanner implements PollingStationScanner{

    private final List<FutureCallback<ScannedData>> callbacks;

    public PollingStationWebScanner() {
        callbacks = new LinkedList<>();
    }

    @Override
    public void subscribe(FutureCallback<ScannedData> scanCallback) {
        callbacks.add(scanCallback);
    }

    @Path(POLLING_STATION_WEB_SCANNER_SCAN_PATH)
    @POST
    @Consumes(MEDIATYPE_PROTOBUF)
    @Produces(MEDIATYPE_PROTOBUF)
    @Override
    public void newScan(ScannedData scannedData) {

        if (callbacks.size() <= 0)
            throw new RuntimeException("No subscribers to forward scan to!");

        for (FutureCallback<ScannedData> callback : callbacks){
            callback.onSuccess(scannedData);
        }

    }

    @Path(POLLING_STATION_WEB_SCANNER_ERROR_PATH)
    @POST
    @Consumes(MEDIATYPE_PROTOBUF)
    @Produces(MEDIATYPE_PROTOBUF)
    @Override
    public void reportScanError(PollingStation.ErrorMsg errorMsg) {

        if (callbacks.size() <= 0)
            throw new RuntimeException("No subscribers to forward error to!");

        for (FutureCallback<ScannedData> callback : callbacks){
            callback.onFailure(new IOException(errorMsg.getMsg()));
        }

    }

    @Path("/test")
    @GET
    public String test(){
        return "test";
    }


}
